@@ -4,11 +4,10 @@ use v5.10;
 use warnings;
 use Moo;
 use Types::Standard qw(HasMethods CodeRef);
-use Carp qw(croak);
 
 use namespace::clean;
 
-our $VERSION = '1.14';
+our $VERSION = '2.00';
 
 has "type" => (
 	is => "ro",
@@ -20,17 +19,8 @@ has "code" => (
 	is => "ro",
 	isa => CodeRef,
 	required => 1,
-	writer => "set_code",
 );
 
-around "BUILDARGS" => sub {
-	my ($orig, $class, @args) = @_;
-
-	croak "Argument to Form::Tiny::Filter->new must be a single arrayref with two elements"
-		unless @args == 1 && ref $args[0] eq ref [] && @{$args[0]} == 2;
-	return {type => $args[0][0], code => $args[0][1]};
-};
-
 sub filter
 {
 	my ($self, $value) = @_;
@@ -48,19 +38,14 @@ __END__
 
 =head1 NAME
 
-Form::Tiny::Filter - a representation of filtering condition
+Form::Tiny::Filter - a representation of a filter
 
 =head1 SYNOPSIS
 
 	# in your form class
-	sub build_filters
-	{
-		return (
-			# the following will be coerced into Form::Tiny::Filter
-			# [ type, filtering sub ]
-			[Str, sub { uc shift() }],
-		);
-	}
+
+	# the following will be coerced into Form::Tiny::Filter
+	form_filer Str, sub { uc shift() };
 
 =head1 DESCRIPTION
 
